import { NgIf, AsyncPipe, NgTemplateOutlet, formatDate } from '@angular/common';
import * as i0 from '@angular/core';
import { InjectionToken, inject, TemplateRef, Directive, Input, Component, ViewEncapsulation, ChangeDetectionStrategy, ContentChildren, LOCALE_ID, NgModule } from '@angular/core';
import { MAT_FORM_FIELD } from '@angular/material/form-field';
import { isObservable, of, pairwise, filter, merge, startWith, map, combineLatest, ReplaySubject } from 'rxjs';
import { switchMap, startWith as startWith$1, map as map$1, distinctUntilChanged } from 'rxjs/operators';
import { ControlContainer, AbstractControl, AbstractControlDirective } from '@angular/forms';
import { coerceArray } from '@angular/cdk/coercion';

/** @docs-private */
function getNgxMatErrorDefMissingForError() {
    return Error(`'for' must be set for ngxMatErrorDef. See example: *ngxMatErrorDef="let error; for: 'pattern'`);
}

const NGX_MAT_ERROR_DEF = new InjectionToken('NGX_MAT_ERROR_DEF');
class NgxMatErrorDef {
    constructor() {
        /**
         * Specify the control to be used for error matching.
         * @optional
         */
        this.ngxMatErrorDefWithControl = undefined;
        this.template = inject(TemplateRef);
        this.controlContainer = inject(ControlContainer, {
            optional: true,
            skipSelf: true,
        });
    }
    get control() {
        const input = this.ngxMatErrorDefWithControl;
        if (typeof input === 'string') {
            return this.controlContainer?.control?.get(input) ?? undefined;
        }
        if (input instanceof AbstractControl) {
            return input;
        }
        return input?.control ?? undefined;
    }
    ngOnInit() {
        if (!this.ngxMatErrorDefFor) {
            throw getNgxMatErrorDefMissingForError();
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorDef, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.0.4", type: NgxMatErrorDef, isStandalone: true, selector: "[ngxMatErrorDef]", inputs: { ngxMatErrorDefFor: "ngxMatErrorDefFor", ngxMatErrorDefWithControl: "ngxMatErrorDefWithControl" }, providers: [
            {
                provide: NGX_MAT_ERROR_DEF,
                useExisting: NgxMatErrorDef,
            },
        ], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorDef, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ngxMatErrorDef]',
                    standalone: true,
                    providers: [
                        {
                            provide: NGX_MAT_ERROR_DEF,
                            useExisting: NgxMatErrorDef,
                        },
                    ],
                }]
        }], propDecorators: { ngxMatErrorDefFor: [{
                type: Input
            }], ngxMatErrorDefWithControl: [{
                type: Input
            }] } });

function coerceToObservable(errorMessages) {
    if (isObservable(errorMessages)) {
        return errorMessages;
    }
    return of(errorMessages);
}

function distinctUntilErrorChanged(prev, curr) {
    if (prev === curr) {
        return true;
    }
    if (!prev || !curr) {
        return false;
    }
    if (prev.template !== curr.template) {
        return false;
    }
    return prev.$implicit === curr.$implicit;
}

/**
 * Find the custom error for a control.
 *
 * @returns INgxMatErrorDef | undefined
 */
function findCustomErrorForControl(errorKeys, customErrorMessages, control) {
    return customErrorMessages.find((customErrorMessage) => {
        return errorKeys.some((error) => {
            if (error !== customErrorMessage.ngxMatErrorDefFor)
                return false;
            return (!customErrorMessage.control || customErrorMessage.control === control);
        });
    });
}

function getAbstractControls(controls) {
    if (!controls) {
        return;
    }
    const _controls = coerceArray(controls)
        .map((control) => !control
        ? undefined
        : control instanceof AbstractControlDirective
            ? control.control
            : control instanceof AbstractControl
                ? control
                : control.ngControl?.control)
        .filter((control) => control != null);
    return _controls.length ? _controls : undefined;
}

function getControlWithError(controls) {
    const controlChanges = controls.map((control) => {
        const fromPendingStates = control.statusChanges.pipe(pairwise(), filter(([previous, current]) => {
            return previous === 'PENDING' && current !== 'PENDING';
        }));
        return merge(control.valueChanges, fromPendingStates).pipe(startWith(null), map(() => control));
    });
    return combineLatest(controlChanges).pipe(map((control) => control.find((control) => !!control.errors)));
}

const NGX_MAT_ERROR_DEFAULT_OPTIONS = new InjectionToken('NGX_MAT_ERROR_DEFAULT_OPTIONS');
class NgxMatErrors {
    constructor() {
        this.messages$ = coerceToObservable(inject(NGX_MAT_ERROR_DEFAULT_OPTIONS));
        this.matFormField = inject(MAT_FORM_FIELD, { optional: true });
        this.controlChangedSubject = new ReplaySubject(1);
    }
    // ContentChildren is set before ngAfterContentInit which is before ngAfterViewInit.
    // Before ngAfterViewInit lifecycle hook we can modify the error$ observable without needing another change detection cycle.
    // This elaborates the need of rxjs defer;
    set customErrorMessages(queryList) {
        const firstControlWithError$ = this.controlChangedSubject.pipe(switchMap((_controls) => {
            const controls = getAbstractControls(_controls || this.matFormField?._control);
            if (!controls) {
                return of(null);
            }
            return getControlWithError(controls);
        })), customErrorMessages$ = queryList.changes.pipe(startWith$1(queryList));
        this.error$ = combineLatest([
            firstControlWithError$,
            customErrorMessages$,
            this.messages$,
        ]).pipe(map$1(([controlWithError, customErrorMessages, messages]) => {
            if (!controlWithError) {
                return;
            }
            const errors = controlWithError.errors;
            const errorKeys = Object.keys(errors);
            const errorOrErrorDef = findCustomErrorForControl(errorKeys, customErrorMessages.toArray(), controlWithError) ?? errorKeys.find((key) => key in messages);
            if (!errorOrErrorDef) {
                return;
            }
            if (typeof errorOrErrorDef === 'object') {
                return {
                    template: errorOrErrorDef.template,
                    $implicit: errors[errorOrErrorDef.ngxMatErrorDefFor],
                };
            }
            const message = messages[errorOrErrorDef];
            return {
                $implicit: typeof message === 'function'
                    ? message(errors[errorOrErrorDef])
                    : message,
            };
        }), distinctUntilChanged(distinctUntilErrorChanged));
    }
    // eslint-disable-next-line @angular-eslint/no-input-rename
    set control(control) {
        this.controlChangedSubject.next(control);
    }
    ngOnDestroy() {
        this.controlChangedSubject.complete();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrors, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.0.4", type: NgxMatErrors, isStandalone: true, selector: "ngx-mat-errors, [ngx-mat-errors]", inputs: { control: ["ngx-mat-errors", "control"] }, host: { classAttribute: "ngx-mat-errors" }, queries: [{ propertyName: "customErrorMessages", predicate: NGX_MAT_ERROR_DEF, descendants: true }], ngImport: i0, template: `<ng-template #defaultTemplate let-error>{{ error }}</ng-template
    ><ng-template [ngIf]="error$ | async" let-error
      ><ng-template
        [ngTemplateOutlet]="error.template ?? defaultTemplate"
        [ngTemplateOutletContext]="error"
      ></ng-template>
    </ng-template>`, isInline: true, dependencies: [{ kind: "directive", type: NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "pipe", type: AsyncPipe, name: "async" }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrors, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngx-mat-errors, [ngx-mat-errors]',
                    template: `<ng-template #defaultTemplate let-error>{{ error }}</ng-template
    ><ng-template [ngIf]="error$ | async" let-error
      ><ng-template
        [ngTemplateOutlet]="error.template ?? defaultTemplate"
        [ngTemplateOutletContext]="error"
      ></ng-template>
    </ng-template>`,
                    encapsulation: ViewEncapsulation.None,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    standalone: true,
                    imports: [NgIf, AsyncPipe, NgTemplateOutlet],
                    host: {
                        class: 'ngx-mat-errors',
                    },
                }]
        }], propDecorators: { customErrorMessages: [{
                type: ContentChildren,
                args: [NGX_MAT_ERROR_DEF, { descendants: true }]
            }], control: [{
                type: Input,
                args: ['ngx-mat-errors']
            }] } });

function errorMessagesEnFactory(locale, format = 'shortDate') {
    return {
        min: (error) => `Please enter a value greater than or equal to ${error.min}.`,
        max: (error) => `Please enter a value less than or equal to ${error.max}.`,
        required: `This field is required.`,
        email: `Please enter a valid email address.`,
        minlength: (error) => `Please enter at least ${error.requiredLength} characters.`,
        maxlength: (error) => `Please enter no more than ${error.requiredLength} characters.`,
        matDatepickerMin: (error) => {
            const formatted = formatDate(error.min, format, locale);
            return `Please enter a date greater than or equal to ${formatted ?? error.min}.`;
        },
        matDatepickerMax: (error) => {
            const formatted = formatDate(error.max, format, locale);
            return `Please enter a date less than or equal to ${formatted ?? error.max}.`;
        },
        matDatepickerParse: (error) => `Invalid date format.`,
        matStartDateInvalid: (error) => `Start date cannot be after end date.`,
        matEndDateInvalid: (error) => `End date cannot be before start date.`,
        matDatepickerFilter: 'This date is filtered out.',
    };
}
const NGX_MAT_ERROR_CONFIG_EN = {
    provide: NGX_MAT_ERROR_DEFAULT_OPTIONS,
    useFactory: errorMessagesEnFactory,
    deps: [LOCALE_ID],
};

function errorMessagesHuFactory(locale, format = 'shortDate') {
    return {
        min: (error) => `Nem lehet kisebb, mint ${error.min}.`,
        max: (error) => `Nem lehet nagyobb, mint ${error.max}.`,
        required: `Kötelező mező.`,
        email: `Nem érvényes e-mail cím.`,
        minlength: (error) => `Legalább ${error.requiredLength} karakter hosszú lehet.`,
        maxlength: (error) => `Legfeljebb ${error.requiredLength} karakter hosszú lehet.`,
        server: (error) => error,
        matDatepickerMin: (error) => {
            const formatted = formatDate(error.min, format, locale);
            // In Hungarian date ends with '.'
            return `Nem lehet korábbi dátum, mint ${formatted ?? error.min}`;
        },
        matDatepickerMax: (error) => {
            const formatted = formatDate(error.max, format, locale);
            // In Hungarian date ends with '.'
            return `Nem lehet későbbi dátum, mint ${formatted ?? error.max}`;
        },
        matDatepickerParse: (error) => `Érvénytelen dátum.`,
        matStartDateInvalid: (error) => `A kezdő dátum nem lehet a vég dátum után.`,
        matEndDateInvalid: (error) => `A vég dátum nem lehe a kezdő dátum előtt.`,
        matDatepickerFilter: 'Ez a dátum nem engedélyezett.',
    };
}
const NGX_MAT_ERROR_CONFIG_HU = {
    provide: NGX_MAT_ERROR_DEFAULT_OPTIONS,
    useFactory: errorMessagesHuFactory,
    deps: [LOCALE_ID],
};

function errorMessagesPtBtFactory(locale, format = 'shortDate') {
    return {
        min: (error) => `Informe um valor igual ou maior a ${error.min}.`,
        max: (error) => `Informe um valor igual ou menor a ${error.max}.`,
        required: `Campo obrigatório.`,
        email: `Informe um endereço de email válido.`,
        minlength: (error) => `Informe pelo menos ${error.requiredLength} caracteres.`,
        maxlength: (error) => `O campo não pode ter mais que ${error.requiredLength} caracteres.`,
        matDatepickerMin: (error) => {
            const formatted = formatDate(error.min, format, locale);
            return `Informe uma data maior ou igual a ${formatted ?? error.min}.`;
        },
        matDatepickerMax: (error) => {
            const formatted = formatDate(error.max, format, locale);
            return `Informe uma data menor ou igual a ${formatted ?? error.max}.`;
        },
        matDatepickerParse: (error) => `Invalid date format.`,
        matStartDateInvalid: (error) => `Start date cannot be after end date.`,
        matEndDateInvalid: (error) => `End date cannot be before start date.`,
        matDatepickerFilter: 'This date is filtered out.',
    };
}
const NGX_MAT_ERROR_CONFIG_PT_BR = {
    provide: NGX_MAT_ERROR_DEFAULT_OPTIONS,
    useFactory: errorMessagesPtBtFactory,
    deps: [LOCALE_ID],
};

class NgxMatErrorsForDateRangePicker {
    constructor() {
        this.ngxMatErrors = inject(NgxMatErrors);
        this.matFormField = inject(MAT_FORM_FIELD);
    }
    ngAfterContentInit() {
        const control = this.matFormField._control;
        this.ngxMatErrors.control = [
            control._startInput.ngControl,
            control._endInput.ngControl,
        ];
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsForDateRangePicker, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.0.4", type: NgxMatErrorsForDateRangePicker, isStandalone: true, selector: "[ngx-mat-errors][forDateRangePicker]", host: { classAttribute: "ngx-mat-errors-for-date-range-picker" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsForDateRangePicker, decorators: [{
            type: Directive,
            args: [{
                    // eslint-disable-next-line @angular-eslint/directive-selector
                    selector: '[ngx-mat-errors][forDateRangePicker]',
                    standalone: true,
                    host: {
                        class: 'ngx-mat-errors-for-date-range-picker',
                    },
                }]
        }] });

class NgxMatErrorsModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsModule, imports: [NgxMatErrors, NgxMatErrorDef], exports: [NgxMatErrors, NgxMatErrorDef] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [NgxMatErrors, NgxMatErrorDef],
                    exports: [NgxMatErrors, NgxMatErrorDef],
                }]
        }] });

/*
 * Public API Surface of ngx-mat-errors
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NGX_MAT_ERROR_CONFIG_EN, NGX_MAT_ERROR_CONFIG_HU, NGX_MAT_ERROR_CONFIG_PT_BR, NGX_MAT_ERROR_DEF, NGX_MAT_ERROR_DEFAULT_OPTIONS, NgxMatErrorDef, NgxMatErrors, NgxMatErrorsForDateRangePicker, NgxMatErrorsModule, errorMessagesEnFactory, errorMessagesHuFactory, errorMessagesPtBtFactory };
//# sourceMappingURL=ngx-mat-errors.mjs.map
